package com.biganto.visual.roompark.conductor

import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.data.service.network.NoNetworkException
import com.biganto.visual.roompark.domain.custom_exception.CustomApiException
import com.biganto.visual.roompark.util.monades.ExceptionString
import com.hannesdorfmann.mosby3.mvi.MviBasePresenter
import com.hannesdorfmann.mosby3.mvp.MvpView
import com.jakewharton.rxrelay2.PublishRelay
import io.reactivex.Observable

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */
abstract class BigantoBasePresenter<V : MvpView, VS>
: MviBasePresenter<V, VS>() {

    protected val restoreStateObservable =
        PublishRelay.create<VS>()

    fun observableParseError(t: Throwable): Observable<VS> =
        Observable.defer { Observable.just(parseError(t)) }


    abstract fun vsByCode(code: Int): (ExceptionString) -> VS
    abstract fun vsByThrowable(t: Throwable): (ExceptionString) -> VS

    open fun parseError(t: Throwable): VS =
        when (t) {
            is CustomApiException -> parseError(t)
            is NoNetworkException -> parseError(t)
            else -> parseError(t)
        }

    private fun parseError(e: CustomApiException) = onCodeReturn(e)
    private fun parseError(e: NoNetworkException) = onNoNetwork(e)
    private fun parseError(e: Exception) = onRandomError(e)

    open fun onRandomError(t: Throwable): VS =
        vsByThrowable(t).invoke(
            ExceptionString(R.string.unknown_error, null)
        )

    open fun onNoNetwork(e: NoNetworkException): VS =
        vsByThrowable(e).invoke(
            ExceptionString(R.string.no_network_error, null)
        )

    private fun onCodeReturn(e: CustomApiException): VS =
        vsByCode(e.code).invoke(ExceptionString(e))

}