package com.biganto.visual.roompark.util.monades

import androidx.annotation.StringRes
import com.biganto.visual.roompark.domain.custom_exception.CustomApiException

/**
 * Created by Vladislav Bogdashkin on 12.11.2019.
 */

sealed class Either<out E, out V> {
    data class Left<out E>(val left: E) : Either<E, Nothing>()
    data class Right<out V>(val right: V) : Either<Nothing, V>()



}


data class ExceptionString(@StringRes private val stringId: Int?, private  val message: String?) {

    val errorMessage : Either<Int,String>

    fun selectHandler(a1:(Int)->Unit,a2:(String)->Unit) =
        when(errorMessage){
            is Either.Left -> a1(errorMessage.left)
            is Either.Right -> a2(errorMessage.right)
        }

    constructor(e:CustomApiException) : this(
        if (e.message!=null) null else e.messageStringId,
        if (e.messageStringId!=null) null else e.message
    )

    init {
        assert(stringId==null && message==null) { "both values cannot be null!" }
        errorMessage = if (stringId!=null) Either.Left(stringId) else Either.Right(message!!)
    }

}
